/*
 *    Amiga Linux/m68k Loader
 *
 *     Copyright 1995 by Geert Uytterhoeven
 *                       (Geert.Uytterhoeven@cs.kuleuven.ac.be)
 *
 *    --------------------------------------------------------------------------
 *
 *    The original idea for this program comes from the Linux distribution:
 *
 *       LILO - Generic Boot Loader for Linux ("LInux LOader")
 *              by Werner Almesberger
 *
 *    --------------------------------------------------------------------------
 *
 *    This file is subject to the terms and conditions of the GNU General Public
 *    License.  See the file README.legal in the main directory of the
 *    Linux/m68k distribution for more details.
 */


%{

#include <string.h>
#include <stdlib.h>
#include <sys/types.h>

#include "lilo.h"
#include "conf.tab.h"


struct keyword {
   char *name;
   int token;
   int value;
};

static struct keyword keywords[] = {
   { "header", HEADER, 0 },
   { "endheader", ENDHEADER, 0 },
   { "bootrec", BOOTREC, 0 },
   { "endrec", ENDREC, 0 },
   { "bootdev", BOOTDEV, 0 },
   { "altdev", ALTDEV, 0 },
   { "default", DEFAULT, 0 },
   { "auto", AUTO, 0 },
   { "timeout", TIMEOUT, 0 },
   { "aux", AUX, 0 },
   { "baud", BAUD, 0 },
   { "password", PASSWORD, 0 },
   { "debug", DEBUG, 0 },
   { "type", TYPE, 0 },
   { "image", IMAGE, 0 },
   { "args", ARGS, 0 },
   { "chipram", CHIPRAM, 0 },
   { "fastram", FASTRAM, 0 },
   { "model", MODEL, 0 },
   { "false", BOOLEAN, FALSE },
   { "true", BOOLEAN, TRUE },
   { "amiga", BOOTOS, BOS_AMIGA },
   { "linux", BOOTOS, BOS_LINUX },
   { "netbsd", BOOTOS, BOS_NETBSD },
   { "A500", AMIMODEL, AMI_500 },
   { "A500+", AMIMODEL, AMI_500PLUS },
   { "A600", AMIMODEL, AMI_600 },
   { "A1000", AMIMODEL, AMI_1000 },
   { "A1200", AMIMODEL, AMI_1200 },
   { "A2000", AMIMODEL, AMI_2000 },
   { "A2500", AMIMODEL, AMI_2500 },
   { "A3000", AMIMODEL, AMI_3000 },
   { "A3000T", AMIMODEL, AMI_3000T },
   { "A3000+", AMIMODEL, AMI_3000PLUS },
   { "A4000", AMIMODEL, AMI_4000 },
   { "A4000T", AMIMODEL, AMI_4000T },
   { "CDTV", AMIMODEL, AMI_CDTV },
   { "CD32", AMIMODEL, AMI_CD32 },
   { "Draco", AMIMODEL, AMI_DRACO },
   { "", -1, 0 }
};

int line = 1;


void yyerror(char *s)
{
   Die("%s:%d: %s\n", ConfigFile, line, s);
}


int yywrap(void)
{
   return(1);
}


int FindToken(char *s)
{
   int i;

   for (i = 0; keywords[i].token > 0; i++)
      if (!strcasecmp(s, keywords[i].name)) {
         yylval = keywords[i].value;
         return(keywords[i].token);
      }
   Die("%s:%d: Unknown keyword `%s'\n", ConfigFile, line, s);
   return(0);
}


char *CopyString(char *s)
{
   int len;
   char *s2;

   len = strlen(s)-2;
   s2 = malloc(len+1);
   strncpy(s2, s+1, len);
   s2[len] = '\0';
   return(s2);
}


%}

keyword  [a-zA-Z][a-zA-Z0-9]*
decnum   [1-9][0-9]*
hexnum   0[xX][0-9a-fA-f]+
octnum   0[0-7]*
number   {decnum}|{hexnum}|{octnum}
string   \"[^\"\n]*\"
comment  \#([^\n]*)
space    [ \t]+
junk     .

%%

{keyword}   {
               return(FindToken(yytext));
            }

{number}    {
               yylval = strtoul(yytext, NULL, 0);
               return(NUMBER);
            }

{string}    {
               yylval = (int)CopyString(yytext);
               return(STRING);
            }

{comment}$  break;

{space}     break;

\n          {
               line++;
               break;
            }

{junk}      {
               Die("%s:%d: Invalid token `%s'\n", ConfigFile, line, yytext);
            }

%%
