/*
 *    Amiga Linux/m68k Loader
 *
 *     Copyright 1995 by Geert Uytterhoeven
 *                       (Geert.Uytterhoeven@cs.kuleuven.ac.be)
 *
 *    --------------------------------------------------------------------------
 *
 *    The original idea for this program comes from the Linux distribution:
 *
 *       LILO - Generic Boot Loader for Linux ("LInux LOader")
 *              by Werner Almesberger
 *
 *    --------------------------------------------------------------------------
 *
 *    This file is subject to the terms and conditions of the GNU General Public
 *    License.  See the file README.legal in the main directory of the
 *    Linux/m68k distribution for more details.
 */


#include <asm/bootinfo.h>


#define VERSION   "Amiga Linux/m68k Loader Version 0.6 (20/2/96)\n" \
                  "(C) Copyright 1995 by Geert Uytterhoeven"


#define LILO_ID               (0x4C494C4F)         /* 'LILO' */
#define LILO_MAPVECTORID      (0x4D415056)         /* 'MAPV' */

#define LILO_BLOCKSIZE        (1024)

#define TRUE                  (1)
#define FALSE                 (0)


#define LILO_CONFIGFILE       "/etc/lilo.conf"
#define LILO_MAPFILE          "/boot/map"
#define LILO_LOADERFILE       "/boot/loader.patched"
#define LILO_LOADERTEMPLATE   "/boot/loader"


#define ID_BOOT_DISK          (0x424F4F55)   /* 'BOOU' */
#define ID_OFS_DISK           (0x444F5300)   /* 'DOS\0' */
#define ID_FFS_DISK           (0x444F5301)   /* 'DOS\1' */
#define ID_OFS_INTL_DISK      (0x444F5302)   /* 'DOS\2' */
#define ID_FFS_INTL_DISK      (0x444F5303)   /* 'DOS\3' */
#define ID_OFS_DC_DISK        (0x444F5304)   /* 'DOS\4' */
#define ID_FFS_DC_DISK        (0x444F5305)   /* 'DOS\5' */
#define ID_MUFS_DISK          (0x6d754653)   /* 'muFS' */
#define ID_MU_OFS_DISK        (0x6d754600)   /* 'muF\0' */
#define ID_MU_FFS_DISK        (0x6d754601)   /* 'muF\1' */
#define ID_MU_OFS_INTL_DISK   (0x6d754602)   /* 'muF\2' */
#define ID_MU_FFS_INTL_DISK   (0x6d754603)   /* 'muF\3' */
#define ID_MU_OFS_DC_DISK     (0x6d754604)   /* 'muF\4' */
#define ID_MU_FFS_DC_DISK     (0x6d754605)   /* 'muF\5' */


struct BootBlock {
   u_long ID;
   u_long Checksum;
   u_long DosBlock;
   u_long Data[252];
   u_long LiloID;
};


   /*
    *    Boot Operating Systems
    */

#define BOS_AMIGA             (0)   /* AmigaOS (always available: 0 == -0 :-) */
#define BOS_LINUX             (1)   /* Linux/m68k */
#define BOS_NETBSD            (2)   /* Future Expansion :-) */


   /*
    *    Map File
    */

#define MAXLABELSIZE          (32)
#define MAXPWDSIZE            (32)


struct MapHeader {
   u_long LiloID;
   u_long Flags;                    /* See definitions below */
   u_long Default;                  /* Sequence Number of Default Boot Image */
   u_long TimeOut;                  /* In Seconds, Zero is Infinite */
   char MasterPwd[MAXPWDSIZE];   
   u_long Baud;                     /* Baud for Auxiliary Console */
                                    /* Zero is ROM default */
};

   /*
    *    Flag definitions
    */

#define LILOF_AUTO            (1)   /* Automatic Boot */
#define LILOF_DEBUG           (2)   /* Loader Debug Mode */
#define LILOF_AUX             (4)   /* Use Auxiliary Console too */


struct MapEntry {
   union {
      u_long NotLast;               /* On Disk */
      struct MapEntry *Next;        /* In Memory */
   } un;
   u_long Size;
   char Label[MAXLABELSIZE];
   char Password[MAXPWDSIZE];
   long Type;                       /* Negative for an Unavailable Boot Image */
   char CommandLine[CL_SIZE];
   u_long ChipSize;                 /* Only valid if not zero */
   u_long FastAddress[NUM_MEMINFO];
   u_long FastSize[NUM_MEMINFO];
   u_int NumFastRAM;
   int Model;
   u_long KernelVector[0];
};

#define arraysize(x)    (sizeof(x)/sizeof(*(x)))


   /*
    *    Alternate Boot Partition
    *
    *    Warning: this definition is hardcoded in bootcode.s too
    */

#define MAXALTDEVSIZE   (32)


   /*
    *    Configuration File Parsing
    */

struct ConfHeader {
   char *BootDevice;
   char *AltDeviceName;
   u_long AltDeviceUnit;
   char *Default;
   u_short Auto;
   u_long TimeOut;
   u_short Aux;
   u_long Baud;
   char *Password;
   u_short Debug;
   unsigned fBootDevice : 1;        /* Flags to check for redefinitions */
   unsigned fAltDevice : 1;
   unsigned fDefault : 1;
   unsigned fAuto : 1;
   unsigned fTimeOut : 1;
   unsigned fAux : 1;
   unsigned fBaud : 1;
   unsigned fPassword : 1;
   unsigned fDebug : 1;
};

struct ConfBoot {
   struct ConfBoot *Next;
   char *Label;
   long Type;
   char *Image;
   char *Args;
   char *Password;
   u_long ChipSize;
   u_long FastAddress[NUM_MEMINFO];
   u_long FastSize[NUM_MEMINFO];
   u_int NumFastRAM;
   int Model;
   unsigned fType : 1;              /* Flags to check for redefinitions */
   unsigned fImage : 1;
   unsigned fArgs : 1;
   unsigned fPassword : 1;
   unsigned fChipRAM : 1;
   unsigned fModel : 1;
};


extern struct ConfHeader ConfHeader;
extern FILE *yyin;
extern char ConfigFile[];

extern void Die(const char *fmt, ...);
extern int AddBootEntry(struct ConfBoot *boot);
extern int yyparse(void);
