/*
 *    Amiga Linux/m68k Loader
 *
 *     Copyright 1995 by Geert Uytterhoeven
 *                       (Geert.Uytterhoeven@cs.kuleuven.ac.be)
 *
 *    --------------------------------------------------------------------------
 *
 *    This program is based on files of the Linux/m68k distribution:
 *
 *     o bootstrap.c -- This program loads the Linux/m68k kernel into an Amiga
 *                      and launches it.
 *
 *       Copyright 1993, 1994 by Hamish Macdonald, Greg Harp
 *
 *       Modified 11-May-94 by Geert Uytterhoeven: A3640 MapROM check
 *       Modified 31-May-94 by Geert Uytterhoeven: Memory thrash problem solved
 *
 *     o bootstrap.h -- This file is a part of the Amiga bootloader.
 *
 *       Copyright 1993, 1994 by Hamish Macdonald
 *
 *       Some minor additions by Michael Rausch 1-11-94
 *       Modified 11-May-94 by Geert Uytterhoeven: inline Supervisor() call
 *
 *
 *    The original idea for this program comes from the Linux distribution:
 *
 *       LILO - Generic Boot Loader for Linux ("LInux LOader")
 *              by Werner Almesberger
 *
 *    --------------------------------------------------------------------------
 *
 *    This file is subject to the terms and conditions of the GNU General Public
 *    License.  See the file README.legal in the main directory of the
 *    Linux/m68k distribution for more details.
 */


   /*
    *    Exec Library Definitions
    */

#define AT_DeadEnd      (0x80000000)
#define AG_NoMemory     (0x00010000)
#define AG_NoSignal     (0x00070000)
#define AG_OpenLib      (0x00030000)
#define AG_OpenDev      (0x00040000)
#define AG_OpenRes      (0x00050000)
#define AO_GraphicsLib  (0x00008002)
#define AO_Intuition    (0x00008004)
#define AO_ExpansionLib (0x0000800A)
#define AO_ConsoleDev   (0x00008011)
#define AO_TimerDev     (0x00008015)
#define AO_Unknown      (0x00008035)

struct MsgPort {
   u_char fill1[15];
   u_char mp_SigBit;
   u_char fill2[18];
};

struct IOStdReq {
   u_char fill1[20];
   struct Device *io_Device;
   u_char fill2[4];
   u_short io_Command;
   u_char io_Flags;
   char io_Error;
   u_long io_Actual;
   u_long io_Length;
   void *io_Data;
   u_char fill4[4];
};

#define CMD_READ        (2)
#define CMD_WRITE       (3)
#define KBD_READMATRIX  (10)
#define IOF_QUICK       (1<<0)

struct timerequest {
   u_char fill1[28];
   u_short io_Command;
   u_char io_Flags;
   u_char fill2[1];
   u_long tv_secs;
   u_long tv_micro;
};

#define UNIT_VBLANK     (1)
#define TR_ADDREQUEST   (9)


#define CONU_SNIPMAP       (3)
#define CONFLAG_DEFAULT    (0)

#define IEQUALIFIER_LSHIFT          (0x0001)
#define IEQUALIFIER_RSHIFT          (0x0002)
#define IEQUALIFIER_CAPSLOCK        (0x0004)
#define IEQUALIFIER_CONTROL         (0x0008)
#define IEQUALIFIER_LALT            (0x0010)
#define IEQUALIFIER_RALT            (0x0020)
#define IEQUALIFIER_LCOMMAND        (0x0040)
#define IEQUALIFIER_RCOMMAND        (0x0080)
#define IEQUALIFIER_MIDBUTTON       (0x1000)
#define IEQUALIFIER_RBUTTON         (0x2000)
#define IEQUALIFIER_LEFTBUTTON      (0x4000)

struct IORequest;
struct Library;


   /*
    *    Intuition Library Definitions
    */

struct NewScreen {
   short LeftEdge;
   short TopEdge;
   short Width;
   short Height;
   short Depth;
   u_char DetailPen;
   u_char BlockPen;
   u_short ViewModes;
   u_short Type;
   struct TextAttr *Font;
   u_char *DefaultTitle;
   struct Gadget *Gadgets;
   struct BitMap *CustomBitMap;
};

#define CUSTOMSCREEN          (0x000f)

struct NewWindow {
   short LeftEdge;
   short TopEdge;
   short Width;
   short Height;
   u_char DetailPen;
   u_char BlockPen;
   u_long IDCMPFlags;
   u_long Flags;
   struct Gadget *FirstGadget;
   struct Image *CheckMark;
   u_char *Title;
   struct Screen *Screen;
   struct BitMap *BitMap;
   short MinWidth;
   short MinHeight;
   u_short MaxWidth;
   u_short MaxHeight;
   u_short Type;
};

#define WFLG_SIMPLE_REFRESH   (0x00000040)
#define WFLG_ACTIVATE         (0x00001000)

struct Window;


   /*
    *    Amiga Shared Library/Device Functions
    */

extern struct ExecBase *SysBase;

#define LVOAbortIO            (-0x1e0)
#define LVOAlert              (-0x6c)
#define LVOCheckIO            (-0x1d4)
#define LVOCloseDevice        (-0x1c2)
#define LVOCloseLibrary       (-0x19e)
#define LVOCreateIORequest    (-0x28e)
#define LVOCreateMsgPort      (-0x29a)
#define LVODebug              (-0x72)
#define LVODeleteIORequest    (-0x294)
#define LVODeleteMsgPort      (-0x2a0)
#define LVODoIO               (-0x1c8)
#define LVOOpenDevice         (-0x1bc)
#define LVOOpenLibrary        (-0x228)
#define LVORawDoFmt           (-0x20a)
#define LVORawMayGetChar      (-0x1fe)
#define LVORawPutChar         (-0x204)
#define LVOSendIO             (-0x1ce)
#define LVOSetFunction        (-0x1a4)
#define LVOWaitIO             (-0x1da)

static __inline char AbortIO(struct IORequest *ioRequest)
{
   register char _res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct IORequest *a1 __asm("%%a1") = ioRequest;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1e0);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a1)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline void Alert(u_long alertNum)
{
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register u_long d7 __asm("%%d7") = alertNum;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x6c);"
                     "exg %%a4,%%a6"
                     : /* no output */
                     : "r" (a4), "r" (d7)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
}

static __inline u_short CheckIO(struct IORequest *ioRequest)
{
   register u_short _res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct IORequest *a1 __asm("%%a1") = ioRequest;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1d4);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a1)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline void CloseDevice(struct IORequest *ioRequest)
{
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct IORequest *a1 __asm("%%a1") = ioRequest;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1c2);"
                     "exg %%a4,%%a6"
                     : /* no output */
                     : "r" (a4), "r" (a1)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
}

static __inline void CloseLibrary(struct Library *library)
{
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct Library *a1 __asm("%%a1") = library;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x19e);"
                     "exg %%a4,%%a6"
                     : /* no output */
                     : "r" (a4), "r" (a1)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
}

static __inline void *CreateIORequest(struct MsgPort *port, u_long size)
{
   register struct Library *_res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct MsgPort *a0 __asm("%%a0") = port;
   register u_long d0 __asm("%%d0") = size;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x28e);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a0), "r" (d0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline struct MsgPort *CreateMsgPort(void)
{
   register struct MsgPort *_res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x29a);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline void DeleteIORequest(void *ioRequest)
{
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register void *a0 __asm("%%a0") = ioRequest;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x294);"
                     "exg %%a4,%%a6"
                     : /* no output */
                     : "r" (a4), "r" (a0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
}

static __inline void DeleteMsgPort(struct MsgPort *port)
{
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct MsgPort *a0 __asm("%%a0") = port;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x2a0);"
                     "exg %%a4,%%a6"
                     : /* no output */
                     : "r" (a4), "r" (a0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
}

static __inline char DoIO(struct IORequest *ioRequest)
{
   register char _res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct IORequest *a1 __asm("%%a1") = ioRequest;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1c8);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a1)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline char OpenDevice(u_char *devName, u_long unit,
                                struct IORequest *ioRequest, u_long flags)
{
   register char _res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register u_char *a0 __asm("%%a0") = devName;
   register u_long d0 __asm("%%d0") = unit;
   register struct IORequest *a1 __asm("%%a1") = ioRequest;
   register u_long d1 __asm("%%d1") = flags;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1bc);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a0), "r" (a1), "r" (d0), "r" (d1)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline struct Library *OpenLibrary(u_char *libName, u_long version)
{
   register struct Library *_res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register u_char *a1 __asm("%%a1") = libName;
   register u_long d0 __asm("%%d0") = version;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x228);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a1), "r" (d0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline void *RawDoFmt(u_char *formatString, void *dataStream,
                               void (*putChProc)(), void *putChData)
{
   register void *_res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register u_char *a0 __asm("%%a0") = formatString;
   register void *a1 __asm("%%a1") = dataStream;
   register void (*a2)() __asm("%%a2") = putChProc;
   register void *a3 __asm("%%a3") = putChData;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x20a);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a0), "r" (a1), "r" (a2), "r" (a3)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline long RawMayGetChar(void)
{
   register long _res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1fe);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline void RawPutChar(char c)
{
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register char d0 __asm("%%d0") = c;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x204);"
                     "exg %%a4,%%a6"
                     : /* no output */
                     : "r" (a4), "r" (d0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
}

static __inline void SendIO(struct IORequest *ioRequest)
{
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct IORequest *a1 __asm("%%a1") = ioRequest;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1ce);"
                     "exg %%a4,%%a6"
                     : /* no output */
                     : "r" (a4), "r" (a1)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
}

static __inline void *SetFunction(struct Library *library, long funcOffset,
                                  void *funcEntry)
{
   register void *_res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct Library *a1 __asm("%%a1") = library;
   register long a0 __asm("%%a0") = funcOffset;
   register void *d0 __asm("%%d0") = funcEntry;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1a4);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a1), "r" (a0), "r" (d0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline char WaitIO(struct IORequest *ioRequest)
{
   register char _res __asm("%%d0");
   register struct ExecBase *a4 __asm("%%a4") = SysBase;
   register struct IORequest *a1 __asm("%%a1") = ioRequest;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x1da);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a1)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}


extern struct InputBase *InputBase;

#define LVOPeekQualifier      (-0x2a)

static __inline u_short PeekQualifier(void)
{
   register u_short _res __asm("%%d0");
   register struct InputBase *a4 __asm("%%a4") = InputBase;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x2a);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}


extern struct IntuitionBase *IntuitionBase;

#define LVOCloseScreen        (-0x42)
#define LVOCloseWindow        (-0x48)
#define LVOOpenScreen         (-0xc6)
#define LVOOpenWindow         (-0xcc)

static __inline u_short CloseScreen(struct Screen *screen)
{
   register u_short _res __asm("%%d0");
   register struct IntuitionBase *a4 __asm("%%a4") = IntuitionBase;
   register struct Screen *a0 __asm("%%a0") = screen;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x42);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline void CloseWindow(struct Window *window)
{
   register struct IntuitionBase *a4 __asm("%%a4") = IntuitionBase;
   register struct Window *a0 __asm("%%a0") = window;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0x48);"
                     "exg %%a4,%%a6"
                     : /* no output */
                     : "r" (a4), "r" (a0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
}

static __inline struct Screen *OpenScreen(struct NewScreen *newScreen)
{
   register struct Screen *_res __asm("%%d0");
   register struct IntuitionBase *a4 __asm("%%a4") = IntuitionBase;
   register struct NewScreen *a0 __asm("%%a0") = newScreen;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0xc6);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}

static __inline struct Window *OpenWindow(struct NewWindow *newWindow)
{
   register struct Window *_res __asm("%%d0");
   register struct IntuitionBase *a4 __asm("%%a4") = IntuitionBase;
   register struct NewWindow *a0 __asm("%%a0") = newWindow;

   __asm __volatile ("exg %%a4,%%a6;"
                     "jsr %%a6@(-0xcc);"
                     "exg %%a4,%%a6"
                     : "=r" (_res)
                     : "r" (a4), "r" (a0)
                     : "%%a0", "%%a1", "%%d0", "%%d1", "memory");
   return(_res);
}
